@@ -1,4 +1,7 @@
 package HTML::GenToc;
+BEGIN {
+  $HTML::GenToc::VERSION = '3.20';
+}
 use strict;
 
 =head1 NAME
@@ -7,12 +10,7 @@ HTML::GenToc - Generate a Table of Contents for HTML documents.
 
 =head1 VERSION
 
-This describes version B<3.10> of HTML::GenToc.
-
-=cut
-
-use vars qw($VERSION);
-$VERSION = '3.10';
+version 3.20
 
 =head1 SYNOPSIS
 
@@ -239,6 +237,21 @@ header => I<file_or_string>
 Either the filename of the file containing header text for ToC;
 or a string containing the header text.
 
+=item ignore_only_one
+
+ignore_only_one => 1
+
+If there would be only one item in the ToC, don't make a ToC.
+
+=item ignore_sole_first
+
+ignore_sole_first => 1
+
+If the first item in the ToC is of the highest level,
+AND it is the only one of that level, ignore it.
+This is useful in web-pages where there is only one H1 header
+but one doesn't know beforehand whether there will be only one.
+
 =item inline
 
 inline => 1
@@ -492,6 +505,7 @@ sub generate_toc ($%) {
 	ol=>$self->{ol},
 	ol_num_levels=>$self->{ol_num_levels},
 	entrysep=>$self->{entrysep},
+	ignore_only_one=>$self->{ignore_only_one},
 	@_
     );
 
@@ -590,51 +604,58 @@ sub generate_toc ($%) {
 	my %labels = ();
 	my @list_of_lists = ();
 	my $i = 0;
-	foreach my $fn (@filenames)
+	for (my $i = 0; $i < @filenames; $i++)
 	{
-	    push @list_of_lists, ($self->make_toc_list(%args,
+	    my @the_list = $self->make_toc_list(%args,
 		first_file=>$filenames[0],
 		labels=>\%labels,
-		filename=>$fn,
-		input=>$input[$i]));
-	    $i++;
-	}
-	#
-	# create the appropriate format
-	#
-	my %formats = ();
-	# check for non-list entries, flagged by negative levels
-	while (my ($key, $val) = each %{$args{toc_entry}})
-	{
-	    if ($val < 0)
+		filename=>$filenames[$i],
+		input=>$input[$i]);
+	    if (!($args{ignore_only_one}
+		and @the_list <= 1))
 	    {
-		$formats{abs($val) - 1} = {};
-		$formats{abs($val) - 1}->{tree_head} = '<ul><li>';
-		$formats{abs($val) - 1}->{tree_foot} = "\n</li></ul>\n";
-		$formats{abs($val) - 1}->{item_sep} = $args{entrysep};
-		$formats{abs($val) - 1}->{pre_item} = '';
-		$formats{abs($val) - 1}->{post_item} = '';
+		push @list_of_lists, @the_list;
 	    }
 	}
-	# check for OL
-	if ($args{ol})
+	if (@list_of_lists > 0)
 	{
-	    $formats{0} = {};
-	    $formats{0}->{tree_head} = '<ol>';
-	    $formats{0}->{tree_foot} = "\n</ol>";
-	    if ($args{ol_num_levels} > 0)
+	    #
+	    # create the appropriate format
+	    #
+	    my %formats = ();
+	    # check for non-list entries, flagged by negative levels
+	    while (my ($key, $val) = each %{$args{toc_entry}})
 	    {
-		$formats{$args{ol_num_levels}} = {};
-		$formats{$args{ol_num_levels}}->{tree_head} = '<ul>';
-		$formats{$args{ol_num_levels}}->{tree_foot} = "\n</ul>";
+		if ($val < 0)
+		{
+		    $formats{abs($val) - 1} = {};
+		    $formats{abs($val) - 1}->{tree_head} = '<ul><li>';
+		    $formats{abs($val) - 1}->{tree_foot} = "\n</li></ul>\n";
+		    $formats{abs($val) - 1}->{item_sep} = $args{entrysep};
+		    $formats{abs($val) - 1}->{pre_item} = '';
+		    $formats{abs($val) - 1}->{post_item} = '';
+		}
 	    }
+	    # check for OL
+	    if ($args{ol})
+	    {
+		$formats{0} = {};
+		$formats{0}->{tree_head} = '<ol>';
+		$formats{0}->{tree_foot} = "\n</ol>";
+		if ($args{ol_num_levels} > 0)
+		{
+		    $formats{$args{ol_num_levels}} = {};
+		    $formats{$args{ol_num_levels}}->{tree_head} = '<ul>';
+		    $formats{$args{ol_num_levels}}->{tree_foot} = "\n</ul>";
+		}
+	    }
+	    $toc_str = HTML::LinkList::link_tree(
+						 %args,
+						 link_tree=>\@list_of_lists,
+						 labels=>\%labels,
+						 formats=>\%formats,
+						);
 	}
-	$toc_str = HTML::LinkList::link_tree(
-	    %args,
-	    link_tree=>\@list_of_lists,
-	    labels=>\%labels,
-	    formats=>\%formats,
-	    );
     }
 
     #
@@ -683,28 +704,21 @@ sub make_anchor_name ($%) {
 	# characters then by the limitations of the values of 'id' and 'name'
 	# attributes: http://www.w3.org/TR/REC-html40/types.html#type-name
         # Eventually, the only punctuation allowed in id values is [_.:-]
-	# Unicode characters with code points > 0x7E (e.g. Chinese characters)
-	# are allowed (test "<h1 id="行政区域">header</h1>" at
-	# http://validator.w3.org/#validate_by_input+with_options), except for
-	# smart quotes (!), see
-	# http://www.w3.org/Search/Mail/Public/search?type-index=www-validator&index-type=t&keywords=[VE][122]+smart+quotes&search=Search+Mail+Archives
-	# However, that contradicts the HTML 4.01 spec: "Anchor names should be
-	# restricted to ASCII characters." -
-	# http://www.w3.org/TR/REC-html40/struct/links.html#h-12.2.1
-	# ...and the [A-Za-z] class of letters mentioned at
-	# http://www.w3.org/TR/REC-html40/types.html#type-name Finally, note
-	# that pod2html fails miserably to generate XHTML-compliant anchor
-	# links. See
-	# http://validator.w3.org/check?uri=http%3A%2F%2Fsearch.cpan.org%2Fdist%2FCatalyst-Runtime%2Flib%2FCatalyst%2FRequest.pm&charset=(detect+automatically)&doctype=XHTML+1.0+Transitional&group=0&user-agent=W3C_Validator%2F1.606
-	$name =~ s/\s/_/g;
+
 	# we need to replace [#&;] only when they are NOT part of an HTML
 	# entity. decode_entities saves us from crafting a nasty regexp
         decode_entities($name);
 	# MediaWiki also uses the period, see
 	# http://en.wikipedia.org/wiki/Hierarchies#Ethics.2C_behavioral_psychology.2C_philosophies_of_identity
-	$name =~ s/([^\w_.:-])/'.'.sprintf('%02X', ord($1))/eg;
+	$name =~ s/([^\s\w_.:-])/'.'.sprintf('%02X', ord($1))/eg;
+
+	$name =~ s/\s+/_/g;
 	# "ID and NAME tokens must begin with a letter ([A-Za-z])"
-        $name = 'L'.$name if $name =~ /\A\W/;
+	$name =~ s/^[^a-zA-Z]+//;
+    }
+    else
+    {
+	$name = 'id';
     }
     $name = 'id' if $name eq '';
 
@@ -969,6 +983,8 @@ sub make_toc_list ($%) {
 	toc_before=>$self->{toc_before},
 	toc_after=>$self->{toc_after},
 	textonly=>$self->{textonly},
+	ignore_sole_first=>$self->{ignore_sole_first},
+	ignore_only_one=>$self->{ignore_only_one},
 	@_
     );
     my $html_str = $args{input};
@@ -978,6 +994,7 @@ sub make_toc_list ($%) {
     my $toc_str = "";
     my @toc = ();
     my @list_of_paths = ();
+    my %level_count = ();
 
     # parse the HTML
     my $hp = new HTML::SimpleParse();
@@ -1135,9 +1152,30 @@ sub make_toc_list ($%) {
 	    path=>$link,
 	    };
 	$labels->{$link} = $content;
+	$level_count{$level}++;
 
 	$name = 'NOTOC';
 	$prevnoli = $noli;
+    } # while tree
+
+    # If we want to ignore the first H1 if there's only one of them 
+    # if the first item is a level-0 item
+    # and there is only one of them
+    # then remove it and readjust levels
+    if ($args{ignore_sole_first}
+	and $level_count{"1"} == 1
+	and $list_of_paths[0]->{level} == 1)
+    {
+	shift @list_of_paths;
+	for (my $i = 0; $i < @list_of_paths; $i++)
+	{
+	    $list_of_paths[$i]->{level}--;
+	}
+    }
+    elsif ($args{ignore_only_one}
+	   and @list_of_paths == 1)
+    {
+	return ();
     }
 
     my @list_of_lists = ();
@@ -1299,66 +1337,72 @@ sub output_toc ($%) {
     my $output = '';
     if ($args{make_toc})
     {
-	my @toc = ();
-	# put the header at the start of the ToC if there is one
-	if ($args{header}) {
-	    if (-f $args{header})
-	    {
-		open(HEADER, $args{header})
-		    || die "Error: unable to open ", $args{header}, ": $!\n";
-		push @toc, <HEADER>;
-		close (HEADER);
+	if ($args{toc})
+	{
+	    my @toc = ();
+	    # put the header at the start of the ToC if there is one
+	    if ($args{header}) {
+		if (-f $args{header})
+		{
+		    open(HEADER, $args{header})
+			|| die "Error: unable to open ", $args{header}, ": $!\n";
+		    push @toc, <HEADER>;
+		    close (HEADER);
+		}
+		else # not a file
+		{
+		    push @toc, $args{header};
+		}
 	    }
-	    else # not a file
-	    {
-		push @toc, $args{header};
+	    # if we are outputing a standalone page,
+	    # then make sure it can stand
+	    elsif (!$args{toc_only}
+		   && !$args{inline}) {
+
+		push @toc, qq|<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML//EN">\n|,
+		     "<html>\n",
+		     "<head>\n";
+		push @toc, "<title>", $args{title}, "</title>\n"  if $args{title};
+		push @toc, "</head>\n",
+		     "<body>\n";
 	    }
-	}
-	# if we are outputing a standalone page,
-	# then make sure it can stand
-	elsif (!$args{toc_only}
-	       && !$args{inline}) {
 
-	    push @toc, qq|<!DOCTYPE HTML PUBLIC "-//W3C//DTD HTML//EN">\n|,
-		 "<html>\n",
-		 "<head>\n";
-	    push @toc, "<title>", $args{title}, "</title>\n"  if $args{title};
-	    push @toc, "</head>\n",
-		 "<body>\n";
-	}
-
-	# start the ToC with the ToC label
-	if ($args{toclabel}) {
-	    push @toc, $args{toclabel};
-	}
-
-	# and the actual ToC
-	push @toc, "\n", $args{toc}, "\n";
-
-	# add the footer, if there is one
-	if ($args{footer}) {
-	    if (-f $args{footer})
-	    {
-		open(FOOTER, $args{footer})
-		    || die "Error: unable to open ", $args{footer}, ": $!\n";
-		push @toc, <FOOTER>;
-		close (FOOTER);
+	    # start the ToC with the ToC label
+	    if ($args{toclabel}) {
+		push @toc, $args{toclabel};
 	    }
-	    else
-	    {
-		push @toc, $args{footer};
+
+	    # and the actual ToC
+	    push @toc, "\n", $args{toc}, "\n";
+
+	    # add the footer, if there is one
+	    if ($args{footer}) {
+		if (-f $args{footer})
+		{
+		    open(FOOTER, $args{footer})
+			|| die "Error: unable to open ", $args{footer}, ": $!\n";
+		    push @toc, <FOOTER>;
+		    close (FOOTER);
+		}
+		else
+		{
+		    push @toc, $args{footer};
+		}
 	    }
+	    # if we are outputing a standalone page,
+	    # then make sure it can stand
+	    elsif (!$args{toc_only}
+		   && !$args{inline}) {
+
+		push @toc, "</body>\n", "</html>\n";
+	    }
+
+	    $output = join '', @toc;
 	}
-	# if we are outputing a standalone page,
-	# then make sure it can stand
-	elsif (!$args{toc_only}
-	       && !$args{inline}) {
-
-	    push @toc, "</body>\n", "</html>\n";
+	else
+	{
+	    $output = "\n";
 	}
-
-	$output = join '', @toc;
-
     }
     elsif ($args{make_anchors} && (!$args{overwrite} || $args{to_string}))
     {
@@ -1884,5 +1928,4 @@ You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-=cut
-
+=cut
\ No newline at end of file
